""" Ajax views for core_explore_keyword_app
"""

from django.contrib import messages
from django.core.exceptions import NON_FIELD_ERRORS
from django.http import JsonResponse
from django.shortcuts import render
from django.views import View

from core_explore_keyword_app.components.search_operator import (
    api as search_operator_api,
)
from core_explore_keyword_app.views.admin.forms import SearchOperatorForm
from core_main_app.commons.exceptions import ApiError


class SearchOperatorConfigModalView(View):
    """View for saving the configuration of a search operator"""

    @staticmethod
    def post(request):
        """Method called after editing or creating a search operator.

        Args:
            request:

        Returns:
        """
        instance = None
        action = "created"
        if "document_id" in request.POST and request.POST["document_id"] != "":
            action = "edited"
            instance = search_operator_api.get_by_id(
                request.POST["document_id"]
            )

        operator_form = SearchOperatorForm(request.POST, instance=instance)

        if not operator_form.is_valid():
            return render(
                request,
                "core_explore_keyword_app/admin/search_ops_manager/modal/config/form.html",
                context={"form": operator_form},
            )

        try:
            operator_form.save()
        except ApiError as exception:
            operator_form.add_error(NON_FIELD_ERRORS, str(exception))
            return render(
                request,
                "core_explore_keyword_app/admin/search_ops_manager/modal/config/form.html",
                context={"form": operator_form},
            )

        messages.add_message(
            request, messages.SUCCESS, "Operator  %s!" % action
        )

        return JsonResponse({})

    @staticmethod
    def get(request):
        """Method called after editing a search operator.

        Args:
            request:

        Returns:
        """
        try:

            operator = search_operator_api.get_by_id(
                request.GET["document_id"]
            )
            xpath_list = "\n".join(operator.xpath_list)
            operator_form = SearchOperatorForm(
                initial={"xpath_list": xpath_list, "document_id": operator.id},
                instance=operator,
            )

            return render(
                request,
                "core_explore_keyword_app/admin/search_ops_manager/modal/config/form.html",
                context={"form": operator_form},
            )
        except ApiError as exception:
            messages.add_message(
                request,
                messages.ERROR,
                "Failed to find operator: %s." % str(exception),
            )


class SearchOperatorDeleteModalView(View):
    """View for deleting a search operator"""

    @staticmethod
    def post(request):
        """Method called after confirmation of the deletion of a search operator.

        Args:
            request:

        Returns:
        """
        if "id" not in request.POST:
            messages.add_message(
                request, messages.ERROR, "Invalid delete request."
            )
            return JsonResponse({}, status=400)

        try:
            operator = search_operator_api.get_by_id(request.POST["id"])
            search_operator_api.delete(operator)

            messages.add_message(
                request,
                messages.SUCCESS,
                "Operator %s deleted." % operator.name,
            )
            return JsonResponse({}, status=200)

        except ApiError as exception:
            messages.add_message(
                request,
                messages.ERROR,
                "Failed to delete operator: %s." % str(exception),
            )
            return JsonResponse({}, status=500)



""" Url router for the explore keyword application
"""

from django.urls import re_path, include

from core_explore_keyword_app.views.user import (
    views as user_views,
    ajax as user_ajax,
)
from core_explore_keyword_app.views.admin import ajax as admin_ajax

urlpatterns = [
    re_path(
        r"^$",
        user_views.KeywordSearchView.as_view(),
        name="core_explore_keyword_app_search",
    ),
    re_path(
        r"^suggestions$",
        user_ajax.SuggestionsKeywordSearchView.as_view(),
        name="core_explore_keyword_suggestions",
    ),
    re_path(
        r"^get-persistent-query-url$",
        user_ajax.CreatePersistentQueryUrlKeywordView.as_view(),
        name="core_explore_keyword_get_persistent_query_url",
    ),
    re_path(
        r"^results-redirect",
        user_views.ResultQueryRedirectKeywordView.as_view(),
        name="core_explore_keyword_results_redirect",
    ),
    re_path(
        r"^(?P<query_id>\w+)$",
        user_views.KeywordSearchView.as_view(),
        name="core_explore_keyword_app_search",
    ),
    re_path(
        r"^operators/delete$",
        admin_ajax.SearchOperatorDeleteModalView.as_view(),
        name="core_explore_keyword_app_search_operator_delete",
    ),
    re_path(
        r"^operators/edit$",
        admin_ajax.SearchOperatorConfigModalView.as_view(),
        name="core_explore_keyword_app_search_operator_edit",
    ),
    re_path(r"^rest/", include("core_explore_keyword_app.rest.urls")),
]



""" Test forms
"""

from unittest import TestCase
from unittest.mock import patch

from django.core.exceptions import ValidationError
from django.test import RequestFactory

from core_explore_keyword_app.forms import KeywordForm
from core_main_app.commons.exceptions import QueryError
from core_main_app.utils.tests_tools.MockUser import create_mock_user


class TestKeywordFormCleanKeywords(TestCase):
    """Test KeywordForm clean_keywords method"""

    def setUp(self) -> None:
        """setUp"""
        self.request = RequestFactory().post("core_explore_keyword_app_search")
        self.request.user = create_mock_user(user_id=1)

        self.keyword = "mock_keyword"
        self.form = KeywordForm(
            request=self.request, data={"keywords": self.keyword}
        )
        self.form.is_valid()

    @patch("core_explore_keyword_app.forms.sanitize_value")
    def test_sanitize_value_function_is_called(self, mock_sanitize_value):
        """test_sanitize_value_function_is_called"""
        self.form.clean_keywords()
        mock_sanitize_value.assert_called_with(self.keyword)

    @patch("core_explore_keyword_app.forms.sanitize_value")
    def test_query_error_raises_validation_error(self, mock_sanitize_value):
        """test_query_error_raises_validation_error"""
        mock_sanitize_value.side_effect = QueryError("mock_query_error")

        with self.assertRaises(ValidationError):
            self.form.clean_keywords()

    @patch("core_explore_keyword_app.forms.sanitize_value")
    def test_sanitize_value_returns_cleaned_data(self, mock_sanitize_value):
        """test_sanitize_value_returns_cleaned_data"""
        mock_sanitize_value.return_value = self.keyword

        self.assertEqual(self.form.clean_keywords(), self.keyword)



""" Discover rules for core explore keyword app.
"""

import logging

from core_explore_keyword_app.permissions import (
    rights as explore_keyword_rights,
)
from core_main_app.permissions import rights as main_rights

logger = logging.getLogger(__name__)


def init_permissions(apps):
    """Initialization of groups and permissions.

    Args:
        apps (Apps): List of applications to init
    """
    try:
        group = apps.get_model("auth", "Group")
        permission = apps.get_model("auth", "Permission")

        # Get or Create the default group
        default_group, created = group.objects.get_or_create(
            name=main_rights.DEFAULT_GROUP
        )

        # Get explore keyword permissions
        explore_access_perm = permission.objects.get(
            codename=explore_keyword_rights.EXPLORE_KEYWORD_ACCESS
        )

        # Add permissions to default group
        default_group.permissions.add(explore_access_perm)
    except Exception as exception:
        logger.error(
            "Impossible to init explore_keyword permissions: %s"
            % str(exception)
        )



""" Apps file for setting core package when app is ready
"""

import sys

from django.apps import AppConfig

from core_explore_keyword_app.permissions import discover


class ExploreKeywordAppConfig(AppConfig):
    """Core application settings"""

    name = "core_explore_keyword_app"
    verbose_name = "Core Explore by Keyword App"

    def ready(self):
        """Run when the app is ready.

        Returns:

        """
        if "migrate" not in sys.argv:
            discover.init_permissions(self.apps)



""" Settings for core_explore_keyword_app

Settings with the following syntax can be overwritten at the project level:
SETTING_NAME = getattr(settings, "SETTING_NAME", "Default Value")
"""

from django.conf import settings

if not settings.configured:
    settings.configure()

INSTALLED_APPS = getattr(settings, "INSTALLED_APPS", [])

# MENU
EXPLORE_MENU_NAME = getattr(settings, "EXPLORE_MENU_NAME", "Query by Keyword")

EXPLORE_KEYWORD_APP_EXTRAS = getattr(
    settings, "EXPLORE_KEYWORD_APP_EXTRAS", []
)



""" Unit Test Persistent Query Keyword
"""

from django.contrib.auth.models import AnonymousUser

from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.utils.integration_tests.integration_base_test_case import (
    IntegrationBaseTestCase,
)
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_explore_common_app.settings import (
    CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT,
)
import core_explore_keyword_app.components.persistent_query_keyword.api as persistent_query_keyword_api

from core_explore_keyword_app.components.persistent_query_keyword.models import (
    PersistentQueryKeyword,
)
from tests.components.persistent_query_keyword.fixtures.fixtures import (
    PersistentQueryKeywordFixtures,
)

fixture_persistent_query_keyword = PersistentQueryKeywordFixtures()


class TestPersistentQueryKeywordGetById(IntegrationBaseTestCase):
    """Test Persistent Query Keyword Get By Id"""

    fixture = fixture_persistent_query_keyword

    def test_get_by_id_as_superuser_returns_persistent_query_keyword(self):
        """test_get_by_id_as_superuser_returns_persistent_query_keyword"""

        # Arrange
        persistent_query_keyword_id = (
            self.fixture.persistent_query_keyword_1.id
        )
        mock_user = create_mock_user("0", is_staff=True, is_superuser=True)

        # Act
        persistent_query_keyword = persistent_query_keyword_api.get_by_id(
            persistent_query_keyword_id, mock_user
        )

        # Assert
        self.assertTrue(
            isinstance(persistent_query_keyword, PersistentQueryKeyword)
        )

    def test_get_by_id_as_owner_returns_persistent_query_keyword(self):
        """test_get_by_id_as_owner_returns_persistent_query_keyword"""

        # Arrange
        persistent_query_keyword_id = (
            self.fixture.persistent_query_keyword_1.id
        )
        mock_user = create_mock_user("1")

        # Act
        persistent_query_keyword = persistent_query_keyword_api.get_by_id(
            persistent_query_keyword_id, mock_user
        )

        # Assert
        self.assertTrue(
            isinstance(persistent_query_keyword, PersistentQueryKeyword)
        )

    def test_get_by_id_as_anonymous_user(self):
        """test_get_by_id_as_anonymous_user"""

        # Arrange
        persistent_query_keyword_id = (
            self.fixture.persistent_query_keyword_1.id
        )

        # Act # Assert
        if CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT:
            self.assertTrue(
                isinstance(
                    persistent_query_keyword_api.get_by_id(
                        persistent_query_keyword_id, AnonymousUser()
                    ),
                    PersistentQueryKeyword,
                )
            )

        else:
            with self.assertRaises(AccessControlError):
                persistent_query_keyword_api.get_by_id(
                    persistent_query_keyword_id, AnonymousUser()
                )

    def test_get_by_id_as_user_not_owner(self):
        """test_get_by_id_as_user_not_owner"""

        # Arrange
        persistent_query_keyword_id = (
            self.fixture.persistent_query_keyword_1.id
        )
        mock_user = create_mock_user("0")

        # Act # Assert

        self.assertTrue(
            isinstance(
                persistent_query_keyword_api.get_by_id(
                    persistent_query_keyword_id, mock_user
                ),
                PersistentQueryKeyword,
            )
        )


class TestPersistentQueryKeywordGetByName(IntegrationBaseTestCase):
    """Test Persistent Query Keyword Get By Name"""

    fixture = fixture_persistent_query_keyword

    def test_get_by_name_as_superuser_returns_persistent_query_keyword(self):
        """test_get_by_name_as_superuser_returns_persistent_query_keyword"""

        # Arrange
        persistent_query_keyword_name = (
            self.fixture.persistent_query_keyword_1.name
        )
        mock_user = create_mock_user("0", is_staff=True, is_superuser=True)

        # Act
        persistent_query_keyword = persistent_query_keyword_api.get_by_name(
            persistent_query_keyword_name, mock_user
        )

        # Assert
        self.assertTrue(
            isinstance(persistent_query_keyword, PersistentQueryKeyword)
        )

    def test_get_by_name_as_owner_returns_persistent_query_keyword(self):
        """test_get_by_name_as_owner_returns_persistent_query_keyword"""

        # Arrange
        persistent_query_keyword_name = (
            self.fixture.persistent_query_keyword_1.name
        )
        mock_user = create_mock_user("1")

        # Act
        persistent_query_keyword = persistent_query_keyword_api.get_by_name(
            persistent_query_keyword_name, mock_user
        )

        # Assert
        self.assertTrue(
            isinstance(persistent_query_keyword, PersistentQueryKeyword)
        )

    def test_get_by_name_as_user_not_owner(self):
        """test_get_by_name_as_user_not_owner"""

        # Arrange
        persistent_query_keyword_name = (
            self.fixture.persistent_query_keyword_1.name
        )
        mock_user = create_mock_user("0")

        # Act # Assert
        self.assertTrue(
            isinstance(
                persistent_query_keyword_api.get_by_name(
                    persistent_query_keyword_name, mock_user
                ),
                PersistentQueryKeyword,
            )
        )

    def test_get_by_name_as_anonymous_user(self):
        """test_get_by_name_as_anonymous_user"""

        # Arrange
        persistent_query_keyword_name = (
            self.fixture.persistent_query_keyword_1.name
        )

        # Act # Assert
        if CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT:
            self.assertTrue(
                isinstance(
                    persistent_query_keyword_api.get_by_name(
                        persistent_query_keyword_name, AnonymousUser()
                    ),
                    PersistentQueryKeyword,
                )
            )

        else:
            with self.assertRaises(AccessControlError):
                persistent_query_keyword_api.get_by_name(
                    persistent_query_keyword_name, AnonymousUser()
                )


class TestPersistentQueryKeywordDelete(IntegrationBaseTestCase):
    """Test Persistent Query Keyword Delete"""

    fixture = fixture_persistent_query_keyword

    def test_delete_others_persistent_query_keyword_as_superuser_deletes_persistent_query_keyword(
        self,
    ):
        """test_delete_others_persistent_query_keyword_as_superuser_deletes_persistent_query_keyword"""

        # Arrange
        persistent_query_keyword = self.fixture.persistent_query_keyword_1
        mock_user = create_mock_user("0", is_staff=True, is_superuser=True)

        # Act
        persistent_query_keyword_api.delete(
            persistent_query_keyword, mock_user
        )

    def test_delete_own_persistent_query_keyword_deletes_persistent_query_keyword(
        self,
    ):
        """test_delete_own_persistent_query_keyword_deletes_persistent_query_keyword"""

        # Arrange
        persistent_query_keyword = self.fixture.persistent_query_keyword_1
        mock_user = create_mock_user("1")

        # Act
        persistent_query_keyword_api.delete(
            persistent_query_keyword, mock_user
        )

    def test_delete_others_persistent_query_keyword_raises_error(self):
        """test_delete_others_persistent_query_keyword_raises_error"""

        # Arrange
        persistent_query_keyword = self.fixture.persistent_query_keyword_1
        mock_user = create_mock_user("0")

        # Act # Assert
        with self.assertRaises(AccessControlError):
            persistent_query_keyword_api.delete(
                persistent_query_keyword, mock_user
            )

    def test_delete_others_user_persistent_query_keyword_as_anonymous_raises_error(
        self,
    ):
        """test_delete_others_user_persistent_query_keyword_as_anonymous_raises_error"""

        # Arrange
        persistent_query_keyword = self.fixture.persistent_query_keyword_1

        # Act # Assert
        with self.assertRaises(AccessControlError):
            persistent_query_keyword_api.delete(
                persistent_query_keyword, AnonymousUser()
            )


class TestPersistentQueryKeywordUpdate(IntegrationBaseTestCase):
    """Test Persistent Query Keyword Update"""

    fixture = fixture_persistent_query_keyword

    def test_update_others_persistent_query_keyword_as_superuser_updates_persistent_query_keyword(
        self,
    ):
        """test_update_others_persistent_query_keyword_as_superuser_updates_persistent_query_keyword"""

        # Arrange
        persistent_query_keyword = self.fixture.persistent_query_keyword_1
        persistent_query_keyword.name = "new_name_persistent_query_keyword_1"
        mock_user = create_mock_user("0", is_staff=True, is_superuser=True)
        # Act
        result = persistent_query_keyword_api.upsert(
            persistent_query_keyword, mock_user
        )
        # Assert
        self.assertTrue(isinstance(result, PersistentQueryKeyword))
        self.assertTrue(result.name, "new_name_persistent_query_keyword_1")

    def test_update_own_persistent_query_keyword_updates_persistent_query_keyword(
        self,
    ):
        """test_update_own_persistent_query_keyword_updates_persistent_query_keyword"""

        # Arrange
        persistent_query_keyword = self.fixture.persistent_query_keyword_1
        mock_user = create_mock_user("1")
        persistent_query_keyword.name = "new_name_persistent_query_keyword_1"
        # Act
        result = persistent_query_keyword_api.upsert(
            persistent_query_keyword, mock_user
        )
        # Assert
        self.assertTrue(isinstance(result, PersistentQueryKeyword))
        self.assertTrue(result.name, "new_name_persistent_query_keyword_1")

    def test_update_others_persistent_query_keyword_raises_error(self):
        """test_update_others_persistent_query_keyword_raises_error"""

        # Arrange
        persistent_query_keyword = self.fixture.persistent_query_keyword_1
        persistent_query_keyword.name = "new_name_persistent_query_keyword_1"
        mock_user = create_mock_user("0")

        # Act # Assert
        with self.assertRaises(AccessControlError):
            persistent_query_keyword_api.upsert(
                persistent_query_keyword, mock_user
            )

    def test_update_others_user_persistent_query_keyword_as_anonymous_raises_error(
        self,
    ):
        """test_update_others_user_persistent_query_keyword_as_anonymous_raises_error"""

        # Arrange
        persistent_query_keyword = self.fixture.persistent_query_keyword_1

        # Act # Assert
        with self.assertRaises(AccessControlError):
            persistent_query_keyword_api.upsert(
                persistent_query_keyword, AnonymousUser()
            )


class TestPersistentQueryKeywordCreate(IntegrationBaseTestCase):
    """Test Persistent Query Keyword Create"""

    fixture = fixture_persistent_query_keyword

    def test_create_others_persistent_query_keyword_as_superuser_creates_persistent_query_keyword(
        self,
    ):
        """test_create_others_persistent_query_keyword_as_superuser_creates_persistent_query_keyword"""

        # Arrange
        persistent_query_keyword = PersistentQueryKeyword(
            name="new_persistent_query_keyword", user_id="0"
        )
        mock_user = create_mock_user("0", is_staff=True, is_superuser=True)
        # Act
        result = persistent_query_keyword_api.upsert(
            persistent_query_keyword, mock_user
        )
        # Assert
        self.assertTrue(isinstance(result, PersistentQueryKeyword))
        self.assertTrue(result.name, "new_persistent_query_keyword")

    def test_create_persistent_query_keyword_as_user_creates_persistent_query_keyword(
        self,
    ):
        """test_create_persistent_query_keyword_as_user_creates_persistent_query_keyword"""

        # Arrange
        persistent_query_keyword = PersistentQueryKeyword(
            name="new_persistent_query_keyword", user_id="1"
        )
        mock_user = create_mock_user("1")

        # Act
        result = persistent_query_keyword_api.upsert(
            persistent_query_keyword, mock_user
        )
        # Assert
        self.assertTrue(isinstance(result, PersistentQueryKeyword))
        self.assertTrue(result.name, "new_persistent_query_keyword")

    def test_create_persistent_query_keyword_as_anonymous_user(self):
        """test_create_persistent_query_keyword_as_anonymous_user"""

        # Arrange
        persistent_query_keyword = PersistentQueryKeyword(
            name="new_persistent_query_keyword", user_id="None"
        )

        # Act
        if CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT:
            result = persistent_query_keyword_api.upsert(
                persistent_query_keyword, AnonymousUser()
            )
            # Assert
            self.assertTrue(isinstance(result, PersistentQueryKeyword))
            self.assertTrue(result.name, "new_persistent_query_keyword")

        else:
            with self.assertRaises(AccessControlError):
                persistent_query_keyword_api.upsert(
                    persistent_query_keyword, AnonymousUser()
                )


class TestPersistentQueryKeywordGetAll(IntegrationBaseTestCase):
    """Test Persistent Query Keyword Get All"""

    fixture = fixture_persistent_query_keyword

    def test_get_all_as_superuser_returns_all_persistent_query_keyword(self):
        """test_get_all_as_superuser_returns_all_persistent_query_keyword"""

        # Arrange
        mock_user = create_mock_user("0", is_staff=True, is_superuser=True)

        # Act
        result = persistent_query_keyword_api.get_all(mock_user)

        # Assert
        self.assertTrue(len(result), 3)

    def test_get_all_as_user_raises_error(self):
        """test_get_all_as_user_raises_error"""

        # Arrange
        mock_user = create_mock_user("1")

        # Assert
        with self.assertRaises(AccessControlError):
            persistent_query_keyword_api.get_all(mock_user)

    def test_get_all_as_anonymous_user_raises_error(self):
        """test_get_all_as_anonymous_user_raises_error"""

        # Assert
        with self.assertRaises(AccessControlError):
            persistent_query_keyword_api.get_all(AnonymousUser())


class TestPersistentQueryKeywordGetAllByUser(IntegrationBaseTestCase):
    """Test Persistent Query Keyword Get All By User"""

    fixture = fixture_persistent_query_keyword

    def test_get_all_by_user_as_superuser_returns_all_user_persistent_query_keyword(
        self,
    ):
        """test_get_all_by_user_as_superuser_returns_all_user_persistent_query_keyword"""

        # Arrange
        mock_user = create_mock_user("1", is_staff=True, is_superuser=True)

        # Act
        result = persistent_query_keyword_api.get_all_by_user(mock_user)

        # Assert
        self.assertTrue(len(result), 1)

    def test_get_all_by_user_returns_all_user_persistent_query_keyword(self):
        """test_get_all_by_user_returns_all_user_persistent_query_keyword"""

        # Arrange
        mock_user = create_mock_user("1")

        # Act
        result = persistent_query_keyword_api.get_all_by_user(mock_user)

        # Assert
        self.assertTrue(len(result), 1)

    def test_get_all_as_anonymous_user_raises_error(self):
        """test_get_all_as_anonymous_user_raises_error"""

        # Assert
        with self.assertRaises(AccessControlError):
            persistent_query_keyword_api.get_all_by_user(AnonymousUser())



""" Unit tests for search operators REST API.
"""

from collections import OrderedDict
from unittest.mock import patch

from django.test import SimpleTestCase
from rest_framework import status
from rest_framework.exceptions import ValidationError

from core_main_app.commons.exceptions import (
    DoesNotExist,
    ModelError,
    NotUniqueError,
)
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock

from core_explore_keyword_app.components.search_operator import (
    api as search_operator_api,
)
from core_explore_keyword_app.components.search_operator.models import (
    SearchOperator,
)
from core_explore_keyword_app.rest.search_operators import (
    views as search_operator_views,
)
from core_explore_keyword_app.rest.search_operators.serializers import (
    SearchOperatorSerializer,
)


class TestSearchOperatorListGet(SimpleTestCase):
    """TestSearchOperatorListGet"""

    def setUp(self) -> None:
        self.mock_user = create_mock_user("1", is_staff=True)
        self.mock_search_operators = [
            SearchOperator(name="mock01", xpath_list=["x/path/a"]),
            SearchOperator(name="mock02", xpath_list=["x/path/b"]),
        ]

    @patch.object(search_operator_api, "get_all")
    def test_get_all_returns_200(self, mock_get_all):
        """test_get_all_returns_200"""

        mock_get_all.return_value = self.mock_search_operators
        response = RequestMock.do_request_get(
            search_operator_views.SearchOperatorList.as_view(), self.mock_user
        )

        self.assertTrue(response.status_code, status.HTTP_200_OK)

    @patch.object(search_operator_api, "get_all")
    def test_get_all_returns_search_operator_list(self, mock_get_all):
        """test_get_all_returns_search_operator_list"""

        mock_get_all.return_value = self.mock_search_operators
        response = RequestMock.do_request_get(
            search_operator_views.SearchOperatorList.as_view(), self.mock_user
        )

        self.assertEqual(
            response.data,
            [
                OrderedDict(
                    {"id": None, "name": "mock01", "xpath_list": ["x/path/a"]}
                ),
                OrderedDict(
                    {"id": None, "name": "mock02", "xpath_list": ["x/path/b"]}
                ),
            ],
        )

    @patch.object(search_operator_api, "get_all")
    def test_empty_list_returns_200(self, mock_get_all):
        """test_empty_list_returns_200"""

        mock_get_all.return_value = []
        response = RequestMock.do_request_get(
            search_operator_views.SearchOperatorList.as_view(), self.mock_user
        )

        self.assertTrue(response.status_code, status.HTTP_200_OK)

    @patch.object(search_operator_api, "get_all")
    @patch.object(SearchOperatorSerializer, "data")
    def test_exception_returns_500(self, mock_serializer_data, mock_get_all):
        """test_exception_returns_500"""

        mock_get_all.side_effect = Exception
        response = RequestMock.do_request_get(
            search_operator_views.SearchOperatorList.as_view(), self.mock_user
        )

        self.assertTrue(
            response.status_code, status.HTTP_500_INTERNAL_SERVER_ERROR
        )


class TestSearchOperatorListPost(SimpleTestCase):
    """Test Search Operator List Post"""

    def setUp(self) -> None:
        self.mock_user = create_mock_user("1", is_staff=True)

    @patch.object(SearchOperatorSerializer, "is_valid")
    @patch.object(SearchOperatorSerializer, "save")
    @patch.object(SearchOperatorSerializer, "data")
    def test_valid_search_operator_returns_201(
        self,
        mock_serializer_data,
        mock_serializer_save,
        mock_serializer_is_valid,
    ):
        """test_valid_search_operator_returns_201"""

        mock_serializer_is_valid.return_value = True
        mock_serializer_save.return_value = None
        mock_serializer_data.return_value = None
        response = RequestMock.do_request_post(
            search_operator_views.SearchOperatorList.as_view(),
            self.mock_user,
            data={
                "name": "mockname",
                "xpath_list": ["/x/path/one", "/x/path/two"],
            },
        )

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    @patch.object(SearchOperatorSerializer, "is_valid")
    def test_invalid_search_operator_returns_400(
        self, mock_serializer_is_valid
    ):
        """test_invalid_search_operator_returns_400"""

        mock_serializer_is_valid.side_effect = ValidationError

        response = RequestMock.do_request_post(
            search_operator_views.SearchOperatorList.as_view(),
            self.mock_user,
            data={
                "name": "mock_name",
                "xpath_list": ["/x/path.one*1..1", "/x/path/two"],
            },
        )

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    @patch.object(SearchOperatorSerializer, "is_valid")
    @patch.object(SearchOperatorSerializer, "save")
    def test_model_error_returns_400(
        self, mock_serializer_save, mock_serializer_is_valid
    ):
        """test_model_error_returns_400"""

        mock_serializer_is_valid.return_value = True
        mock_serializer_save.side_effect = ModelError(message="mock error")

        response = RequestMock.do_request_post(
            search_operator_views.SearchOperatorList.as_view(),
            self.mock_user,
            data={
                "name": "mock_name",
                "xpath_list": ["/x/path.one*1..1", "/x/path/two"],
            },
        )

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    @patch.object(SearchOperatorSerializer, "is_valid")
    @patch.object(SearchOperatorSerializer, "save")
    def test_duplicate_returns_400(
        self, mock_serializer_save, mock_serializer_is_valid
    ):
        """test_duplicate_returns_400"""

        mock_serializer_is_valid.return_value = True
        mock_serializer_save.side_effect = NotUniqueError(message="mock error")
        response = RequestMock.do_request_post(
            search_operator_views.SearchOperatorList.as_view(),
            self.mock_user,
            data={
                "name": "mock_name",
                "xpath_list": ["/x/path.one*1..1", "/x/path/two"],
            },
        )

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    @patch.object(SearchOperatorSerializer, "__init__")
    def test_key_error_returns_400(self, mock_serializer):
        """test_key_error_returns_400"""

        mock_serializer.side_effect = KeyError
        response = RequestMock.do_request_post(
            search_operator_views.SearchOperatorList.as_view(),
            self.mock_user,
            data={
                "name": "mock_name",
            },
        )

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    @patch.object(SearchOperatorSerializer, "__init__")
    def test_exception_returns_500(self, mock_serializer):
        """test_exception_returns_500"""

        mock_serializer.side_effect = Exception
        response = RequestMock.do_request_post(
            search_operator_views.SearchOperatorList.as_view(),
            self.mock_user,
            data={
                "name": "mock_name",
            },
        )

        self.assertEqual(
            response.status_code, status.HTTP_500_INTERNAL_SERVER_ERROR
        )


class TestSearchOperatorDetailGet(SimpleTestCase):
    """Test Search Operator Detail Get"""

    def setUp(self) -> None:
        self.mock_user = create_mock_user("1", is_staff=True)

    @patch.object(search_operator_api, "get_by_id")
    def test_default_returns_200(self, mock_get_by_id):
        """test_default_returns_200"""

        mock_get_by_id.return_value = {
            "name": "mock_operator",
            "xpath_list": ["/x/path/a", "/x/path/b"],
        }
        response = RequestMock.do_request_get(
            search_operator_views.SearchOperatorDetail.as_view(),
            self.mock_user,
            param={"pk": 1},
        )

        self.assertTrue(response.status_code, status.HTTP_200_OK)

    @patch.object(search_operator_api, "get_by_id")
    def test_default_returns_search_operator_data(self, mock_get_by_id):
        """test_default_returns_search_operator_data"""

        search_operator = {
            "name": "mock_operator",
            "xpath_list": ["/x/path/a", "/x/path/b"],
        }
        mock_get_by_id.return_value = search_operator
        response = RequestMock.do_request_get(
            search_operator_views.SearchOperatorDetail.as_view(),
            self.mock_user,
            param={"pk": 1},
        )

        self.assertEqual(response.data, search_operator)

    @patch.object(search_operator_api, "get_by_id")
    def test_nonexistent_returns_404(self, mock_get_by_id):
        """test_nonexistent_returns_404"""

        mock_get_by_id.side_effect = DoesNotExist(message="mock error")
        response = RequestMock.do_request_get(
            search_operator_views.SearchOperatorDetail.as_view(),
            self.mock_user,
            param={"pk": 1},
        )

        self.assertTrue(response.status_code, status.HTTP_404_NOT_FOUND)

    @patch.object(search_operator_api, "get_by_id")
    def test_exception_returns_500(self, mock_get_by_id):
        """test_exception_returns_500"""

        mock_get_by_id.side_effect = Exception
        response = RequestMock.do_request_get(
            search_operator_views.SearchOperatorDetail.as_view(),
            self.mock_user,
            param={"pk": 1},
        )

        self.assertTrue(
            response.status_code, status.HTTP_500_INTERNAL_SERVER_ERROR
        )


class TestSearchOperatorDetailPatch(SimpleTestCase):
    """Test Search Operator Detail Patch"""

    def setUp(self) -> None:
        self.mock_user = create_mock_user("1", is_staff=True)

    @patch.object(search_operator_api, "get_by_id")
    def test_valid_operator_returns_200(self, mock_get_by_id):
        """test_valid_operator_returns_200"""

        mock_get_by_id.return_value = {
            "name": "mock_operator",
            "xpath_list": ["/x/path/a", "/x/path/b"],
        }
        response = RequestMock.do_request_patch(
            search_operator_views.SearchOperatorDetail.as_view(),
            self.mock_user,
            param={"pk": 1},
        )

        self.assertTrue(response.status_code, status.HTTP_200_OK)

    @patch.object(search_operator_api, "get_by_id")
    def test_valid_operator_returns_operator(self, mock_get_by_id):
        """test_valid_operator_returns_operator"""

        mock_operator = {
            "name": "mock_operator",
            "xpath_list": ["/x/path/a", "/x/path/b"],
        }
        mock_get_by_id.return_value = mock_operator
        response = RequestMock.do_request_patch(
            search_operator_views.SearchOperatorDetail.as_view(),
            self.mock_user,
            param={"pk": 1},
        )

        self.assertTrue(response.data, mock_operator)

    @patch.object(SearchOperatorSerializer, "is_valid")
    def test_invalid_operator_returns_400(self, mock_serializer_is_valid):
        """test_invalid_operator_returns_400"""

        mock_serializer_is_valid.side_effect = ValidationError
        response = RequestMock.do_request_patch(
            search_operator_views.SearchOperatorDetail.as_view(),
            self.mock_user,
            param={"pk": 1},
        )

        self.assertTrue(response.status_code, status.HTTP_400_BAD_REQUEST)

    @patch.object(search_operator_api, "get_by_id")
    def test_nonexistent_operator_returns_404(self, mock_get_by_id):
        """test_nonexistent_operator_returns_404"""

        mock_get_by_id.side_effect = DoesNotExist
        response = RequestMock.do_request_patch(
            search_operator_views.SearchOperatorDetail.as_view(),
            self.mock_user,
            param={"pk": 1},
        )

        self.assertTrue(response.status_code, status.HTTP_404_NOT_FOUND)

    @patch.object(SearchOperatorSerializer, "save")
    def test_exception_returns_500(self, mock_serializer_save):
        """test_exception_returns_500"""

        mock_serializer_save.side_effect = Exception
        response = RequestMock.do_request_patch(
            search_operator_views.SearchOperatorDetail.as_view(),
            self.mock_user,
            param={"pk": 1},
        )

        self.assertTrue(
            response.status_code, status.HTTP_500_INTERNAL_SERVER_ERROR
        )


class TestSearchOperatorDetailDelete(SimpleTestCase):
    """Test Search Operator Detail Delete"""

    def setUp(self) -> None:
        self.mock_user = create_mock_user("1", is_staff=True)

    @patch.object(search_operator_api, "get_by_id")
    @patch.object(search_operator_api, "delete")
    def test_valid_returns_204(self, mock_delete, mock_get_by_id):
        """test_valid_returns_204"""

        mock_delete.return_value = None
        mock_get_by_id.return_value = {}
        response = RequestMock.do_request_delete(
            search_operator_views.SearchOperatorDetail.as_view(),
            self.mock_user,
            param={"pk": 1},
        )

        self.assertTrue(response.status_code, status.HTTP_204_NO_CONTENT)

    @patch.object(search_operator_api, "get_by_id")
    def test_nonexistent_returns_404(self, mock_get_by_id):
        """test_nonexistent_returns_404"""

        mock_get_by_id.side_effect = DoesNotExist
        response = RequestMock.do_request_delete(
            search_operator_views.SearchOperatorDetail.as_view(),
            self.mock_user,
            param={"pk": 1},
        )

        self.assertTrue(response.status_code, status.HTTP_404_NOT_FOUND)

    @patch.object(search_operator_api, "get_by_id")
    @patch.object(search_operator_api, "delete")
    def test_exception_returns_500(self, mock_delete, mock_get_by_id):
        """test_exception_returns_500"""

        mock_get_by_id.return_value = {}
        mock_delete.side_effect = Exception
        response = RequestMock.do_request_delete(
            search_operator_views.SearchOperatorDetail.as_view(),
            self.mock_user,
            param={"pk": 1},
        )

        self.assertTrue(
            response.status_code, status.HTTP_500_INTERNAL_SERVER_ERROR
        )



""" Unit tests for SearchOperator API calls.
"""

from unittest import TestCase, mock

from core_main_app.commons import exceptions
from core_explore_keyword_app.components.search_operator import (
    api as search_operator_api,
)
from core_explore_keyword_app.components.search_operator.models import (
    SearchOperator,
)


class TestsApiGetAll(TestCase):
    """Tests Api Get All"""

    @mock.patch.object(SearchOperator, "get_all")
    def test_returns_no_error(self, mock_get_all):
        """test_returns_no_error"""

        expected_result = ["search_operator_1", "search_operator_2"]
        mock_get_all.return_value = expected_result

        self.assertListEqual(search_operator_api.get_all(), expected_result)


class TestsApiGetById(TestCase):
    """Tests Api Get By Id"""

    @mock.patch.object(SearchOperator, "get_by_id")
    def test_returns_no_error(self, mock_get_by_id):
        """test_returns_no_error"""

        expected_result = "search_operator"
        mock_get_by_id.return_value = expected_result

        self.assertEqual(
            search_operator_api.get_by_id("mock_id"), expected_result
        )

    @mock.patch.object(SearchOperator, "get_by_id")
    def test_incorrect_id_raises_api_error(self, mock_get_by_id):
        """test_incorrect_id_raises_api_error"""

        mock_get_by_id.side_effect = exceptions.ModelError(
            message="mock error"
        )

        with self.assertRaises(exceptions.ApiError):
            search_operator_api.get_by_id("mock_id")

    @mock.patch.object(SearchOperator, "get_by_id")
    def test_nonexistant_raises_api_error(self, mock_get_by_id):
        """test_nonexistant_raises_api_error"""

        mock_get_by_id.side_effect = exceptions.DoesNotExist(
            message="mock error"
        )

        with self.assertRaises(exceptions.ApiError):
            search_operator_api.get_by_id("mock_id")


class TestsApiGetByName(TestCase):
    """Tests Api Get By Name"""

    @mock.patch.object(SearchOperator, "get_by_name")
    def test_returns_no_error(self, mock_get_by_name):
        """test_returns_no_error"""

        expected_result = "search_operator"
        mock_get_by_name.return_value = expected_result

        self.assertEqual(
            search_operator_api.get_by_name("mock_name"), expected_result
        )

    @mock.patch.object(SearchOperator, "get_by_name")
    def test_nonexistant_raises_api_error(self, mock_get_by_name):
        """test_nonexistant_raises_api_error"""

        mock_get_by_name.side_effect = exceptions.DoesNotExist(
            message="mock error"
        )

        with self.assertRaises(exceptions.ApiError):
            search_operator_api.get_by_name("mock_name")


class TestsApiGetByDotNotationList(TestCase):
    """Tests Api Get By Dot Notation List"""

    @mock.patch.object(SearchOperator, "get_by_dot_notation_list")
    def test_returns_no_error(self, mock_get_by_dot_notation_list):
        """test_returns_no_error"""

        expected_result = "search_operator"
        mock_get_by_dot_notation_list.return_value = expected_result

        self.assertEqual(
            search_operator_api.get_by_dot_notation_list(
                ["dot_not_1", "dot_not_2"]
            ),
            expected_result,
        )

    @mock.patch.object(SearchOperator, "get_by_dot_notation_list")
    def test_nonexistant_raises_api_error(self, mock_get_by_dot_notation_list):
        """test_nonexistant_raises_api_error"""

        mock_get_by_dot_notation_list.side_effect = exceptions.DoesNotExist(
            message="mock error"
        )

        with self.assertRaises(exceptions.ApiError):
            search_operator_api.get_by_dot_notation_list(
                ["dot_not_1", "dot_not_2"]
            )


class TestsApiUpsert(TestCase):
    """Tests Api Upsert"""

    def setUp(self) -> None:
        self.mock_search_operator = SearchOperator(
            name="mock_operator", xpath_list=["/x/path/a", "/x/path/b"]
        )

    @mock.patch.object(SearchOperator, "save")
    def test_returns_no_error(self, mock_save):
        """test_returns_no_error"""

        mock_save.return_value = None

        search_operator_api.upsert(self.mock_search_operator)

    @mock.patch.object(SearchOperator, "save")
    def test_dot_notation_list_is_correct(self, mock_save):
        """test_dot_notation_list_is_correct"""

        expected_dot_notation_list = ["x.path.a", "x.path.b"]
        mock_save.return_value = self.mock_search_operator

        self.assertListEqual(
            search_operator_api.upsert(
                self.mock_search_operator
            ).dot_notation_list,
            expected_dot_notation_list,
        )

    @mock.patch.object(SearchOperator, "save")
    def test_duplicate_raises_api_error(self, mock_save):
        """test_duplicate_raises_api_error"""

        mock_save.side_effect = exceptions.NotUniqueError(message="mock error")

        with self.assertRaises(exceptions.ApiError):
            search_operator_api.upsert(self.mock_search_operator)


class TestsApiDelete(TestCase):
    """Tests Api Delete"""

    @mock.patch.object(SearchOperator, "delete")
    def test_returns_no_error(self, mock_delete):
        """test_returns_no_error"""

        mock_delete.return_value = None

        self.assertEqual(search_operator_api.delete(SearchOperator()), None)



""" Custom admin site for the Persistent Query Keyword model
"""

from django.contrib import admin


class CustomPersistentQueryKeywordAdmin(admin.ModelAdmin):
    """CustomPersistentQueryKeywordAdmin"""

    exclude = ["data_sources"]

    def has_add_permission(self, request, obj=None):
        """Prevent from manually adding Persistent Queries"""
        return False



""" Persistent Query Keyword API
"""

from core_main_app.access_control.api import has_perm_administration
from core_main_app.access_control.decorators import access_control
from core_explore_common_app.access_control.api import (
    can_read_persistent_query,
    can_write_persistent_query,
)
from core_explore_keyword_app.components.persistent_query_keyword.models import (
    PersistentQueryKeyword,
)


@access_control(can_write_persistent_query)
def upsert(persistent_query_keyword, user):
    """Saves or update persistent query

    Args:
        persistent_query_keyword:
        user:

    Returns:

    """
    persistent_query_keyword.save()
    return persistent_query_keyword


@access_control(can_read_persistent_query)
def get_by_id(persistent_query_keyword_id, user):
    """Return the Persistent Query Keyword with the given id

    Args:
        persistent_query_keyword_id:
        user:
    Returns:

    """
    return PersistentQueryKeyword.get_by_id(persistent_query_keyword_id)


@access_control(can_read_persistent_query)
def get_by_name(persistent_query_keyword_name, user):
    """Return the Persistent Query Keyword with the given name

    Args:
        persistent_query_keyword_name:
        user:
    Returns:

    """
    return PersistentQueryKeyword.get_by_name(persistent_query_keyword_name)


@access_control(can_write_persistent_query)
def delete(persistent_query_keyword, user):
    """Deletes the Persistent Query Keyword

    Args:
        persistent_query_keyword:
        user:
    """
    persistent_query_keyword.delete()


@access_control(can_write_persistent_query)
def set_name(persistent_query_keyword, name, user):
    """Set name to Persistent Query Keyword

    Args:
        persistent_query_keyword:
        name:
        user:
    """
    persistent_query_keyword.name = name
    persistent_query_keyword.save()


@access_control(has_perm_administration)
def get_all(user):
    """get all Persistent Query Keyword

    Args:
        user:
    """
    return PersistentQueryKeyword.get_all()


@access_control(can_read_persistent_query)
def get_all_by_user(user):
    """get persistent Query Keyword by user

    Args:
        user:
    """
    return PersistentQueryKeyword.get_all_by_user(user.id)



"""Explore keyword app Ajax views
"""

import json
import logging
import re

from django.http import HttpResponse, HttpResponseBadRequest
from django.utils.decorators import method_decorator
from django.views.generic import View

import core_main_app.components.template_version_manager.api as template_version_manager_api
from core_explore_common_app.components.query import api as query_api
from core_explore_common_app.rest.query.views import execute_local_query
from core_explore_common_app.utils.query.query import (
    create_local_data_source,
    serialize_query,
    is_local_data_source,
)
from core_explore_common_app.views.user.ajax import (
    CreatePersistentQueryUrlView,
)
from core_explore_keyword_app.components.persistent_query_keyword.models import (
    PersistentQueryKeyword,
)
from core_explore_keyword_app.forms import KeywordForm
from core_explore_keyword_app.permissions import rights
from core_main_app.components.template import api as template_api
from core_main_app.utils import decorators
from core_main_app.utils.databases.mongo.pymongo_database import (
    get_full_text_query,
)
from core_main_app.utils.query.mongo.prepare import sanitize_value

logger = logging.getLogger("core_explore_keyword_app.views.user.ajax")


def _get_local_data_source(query):
    """Check if there is a data source that is local.

    Args:
        query:

    Returns:
    """
    # If we find a data source that is local
    for data_source in query.data_sources:
        # find local data source
        if is_local_data_source(data_source):
            return data_source
    return None


class SuggestionsKeywordSearchView(View):
    """Suggestions Keyword Search View"""

    @method_decorator(
        decorators.permission_required(
            content_type=rights.EXPLORE_KEYWORD_CONTENT_TYPE,
            permission=rights.EXPLORE_KEYWORD_ACCESS,
        )
    )
    def post(self, request, *args, **kwargs):
        """POST

        Args:
            request:
            *args:
            **kwargs:

        Returns:

        """
        suggestions = []
        search_form = KeywordForm(data=request.POST, request=request)
        keywords = request.POST.get("term")

        if not search_form.is_valid():
            error_message = (
                "Exception while generating suggestions: Search form is "
                "invalid"
            )
            logger.error(error_message)
            return HttpResponseBadRequest(error_message)

        try:
            sanitize_value(keywords)  # Ensure only valid keywords are passed

            # get form values
            query_id = search_form.cleaned_data.get("query_id", None)
            global_templates = search_form.cleaned_data.get(
                "global_templates", []
            )
            user_templates = search_form.cleaned_data.get("user_templates", [])

            # get all template version manager ids
            template_version_manager_ids = global_templates + user_templates

            # from ids, get all version manager
            version_manager_list = template_version_manager_api.get_by_id_list(
                template_version_manager_ids, request=request
            )

            # from all version manager, build a list of all version (template)
            template_ids = []
            list(
                [template_ids.extend(x.versions) for x in version_manager_list]
            )

            if query_id is not None and keywords is not None:
                # get query
                query = query_api.get_by_id(query_id, request.user)

                # Get local data source
                local_data_source = _get_local_data_source(query)

                if local_data_source:

                    # Prepare query
                    query = self._get_query_prepared(
                        keywords, query, request, template_ids
                    )

                    # Send query
                    json_query = serialize_query(query, local_data_source)
                    dict_results = execute_local_query(json_query, 1, request)

                    if dict_results.paginator.count > 0:
                        self._extract_suggestion_from_results(
                            dict_results.object_list, keywords, suggestions
                        )

            return HttpResponse(
                json.dumps({"suggestions": suggestions}),
                content_type="application/javascript",
            )
        except Exception as exception:
            error_message = (
                "Exception while generating suggestions: %s",
                str(exception),
            )
            logger.error(error_message)
            return HttpResponseBadRequest(error_message)

    def _get_query_prepared(self, keywords, query, request, template_ids):
        """Prepare the query for suggestions.

        Args:
            keywords:
            query:
            request:
            template_ids:
        Returns:
        """

        # update query
        query.templates.set(
            template_api.get_all_accessible_by_id_list(
                template_ids, request=request
            )
        )
        # TODO: improve query to get better results
        query.content = json.dumps(get_full_text_query(keywords))
        # Data source is local
        query.data_sources = [create_local_data_source(request)]
        return query

    @staticmethod
    def _extract_suggestion_from_results(results, keywords, suggestions):
        """Extract suggestion from

        Args:
            results:
            keywords:
            suggestions:

        Returns:
        """
        # Prepare keywords
        word_list = re.sub(r"[^\w]", " ", keywords).split()
        word_list = [x + "|" + x + r"\w+" for x in word_list]
        word_list = "|".join(word_list)
        for result in results:
            # Extract suggestions from data
            list_whole_keywords = re.findall(
                "\\b(" + word_list + ")\\b",
                result.content,
                flags=re.IGNORECASE,
            )
            labels = list(set(list_whole_keywords))

            for label in labels:
                label = label.lower()
                result_json = dict()
                result_json["label"] = label
                result_json["value"] = label
                if result_json not in suggestions:
                    suggestions.append(result_json)


class CreatePersistentQueryUrlKeywordView(CreatePersistentQueryUrlView):
    """Create the persistent url from a Query"""

    view_to_reverse = "core_explore_keyword_results_redirect"

    @staticmethod
    def _create_persistent_query(query):
        # create the persistent query
        return PersistentQueryKeyword(
            user_id=query.user_id,
            content=query.content,
        )



""" Authentication tests for search operators REST API.
"""

from unittest.mock import patch

from django.test import SimpleTestCase
from rest_framework import status

from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock
import core_explore_keyword_app.components.search_operator.api as search_operator_api
from core_explore_keyword_app.rest.search_operators import (
    views as search_operator_rest_views,
)
from core_explore_keyword_app.rest.search_operators.serializers import (
    SearchOperatorSerializer,
)


class TestSearchOperatorListGet(SimpleTestCase):
    """Test Search Operator List Get"""

    @patch.object(search_operator_api, "get_all")
    def test_anonymous_returns_http_200(
        self, mock_search_operator_api_get_all
    ):
        """test_anonymous_returns_http_200"""

        mock_search_operator_api_get_all.return_value = []
        response = RequestMock.do_request_get(
            search_operator_rest_views.SearchOperatorList.as_view(), None
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(search_operator_api, "get_all")
    def test_authenticated_returns_http_200(
        self, mock_search_operator_api_get_all
    ):
        """test_authenticated_returns_http_200"""

        mock_user = create_mock_user("1")
        mock_search_operator_api_get_all.return_value = []

        response = RequestMock.do_request_get(
            search_operator_rest_views.SearchOperatorList.as_view(), mock_user
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(search_operator_api, "get_all")
    @patch.object(SearchOperatorSerializer, "data")
    def test_staff_returns_http_200(
        self,
        mock_search_operator_serializer_data,
        mock_search_operator_api_get_all,
    ):
        """test_staff_returns_http_200"""

        mock_search_operator_serializer_data.return_value = {}
        mock_search_operator_api_get_all.return_value = []
        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_get(
            search_operator_rest_views.SearchOperatorList.as_view(), mock_user
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestSearchOperatorListPost(SimpleTestCase):
    """Test Search Operator List Post"""

    def test_anonymous_returns_http_403(self):
        """test_anonymous_returns_http_403"""

        response = RequestMock.do_request_post(
            search_operator_rest_views.SearchOperatorList.as_view(), None
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_authenticated_returns_http_403(self):
        """test_authenticated_returns_http_403"""

        mock_user = create_mock_user("1")

        response = RequestMock.do_request_post(
            search_operator_rest_views.SearchOperatorList.as_view(), mock_user
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(SearchOperatorSerializer, "is_valid")
    @patch.object(SearchOperatorSerializer, "save")
    @patch.object(SearchOperatorSerializer, "data")
    def test_staff_returns_http_201(
        self,
        mock_search_operator_serializer_data,
        mock_search_operator_serializer_save,
        mock_search_operator_serializer_is_valid,
    ):
        """test_staff_returns_http_201"""

        mock_search_operator_serializer_data.return_value = {}
        mock_search_operator_serializer_save.return_value = None
        mock_search_operator_serializer_is_valid.return_value = True
        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_post(
            search_operator_rest_views.SearchOperatorList.as_view(), mock_user
        )

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)


class TestSearchOperatorDetailGet(SimpleTestCase):
    """Test Search Operator Detail Get"""

    @patch.object(search_operator_api, "get_by_id")
    @patch.object(SearchOperatorSerializer, "data")
    def test_anonymous_returns_http_200(
        self,
        mock_search_operator_serializer_data,
        mock_search_operator_api_get_by_id,
    ):
        """test_anonymous_returns_http_200"""

        mock_search_operator_serializer_data.return_value = {}
        mock_search_operator_api_get_by_id.return_value = None
        response = RequestMock.do_request_get(
            search_operator_rest_views.SearchOperatorDetail.as_view(),
            None,
            param={"pk": 0},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(search_operator_api, "get_by_id")
    @patch.object(SearchOperatorSerializer, "data")
    def test_authenticated_returns_http_200(
        self,
        mock_search_operator_serializer_data,
        mock_search_operator_api_get_by_id,
    ):
        """test_authenticated_returns_http_200"""

        mock_search_operator_serializer_data.return_value = {}
        mock_search_operator_api_get_by_id.return_value = None
        mock_user = create_mock_user("1")

        response = RequestMock.do_request_get(
            search_operator_rest_views.SearchOperatorDetail.as_view(),
            mock_user,
            param={"pk": 0},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(search_operator_api, "get_by_id")
    @patch.object(SearchOperatorSerializer, "data")
    def test_staff_returns_http_200(
        self,
        mock_search_operator_serializer_data,
        mock_search_operator_api_get_by_id,
    ):
        """test_staff_returns_http_200"""

        mock_search_operator_serializer_data.return_value = {}
        mock_search_operator_api_get_by_id.return_value = None
        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_get(
            search_operator_rest_views.SearchOperatorDetail.as_view(),
            mock_user,
            param={"pk": 0},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestSearchOperatorDetailPatch(SimpleTestCase):
    """Test Search Operator Detail Patch"""

    def test_anonymous_returns_http_403(self):
        """test_anonymous_returns_http_403"""
        response = RequestMock.do_request_patch(
            search_operator_rest_views.SearchOperatorDetail.as_view(),
            None,
            param={"pk": 0},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_authenticated_returns_http_403(self):
        """test_authenticated_returns_http_403"""

        mock_user = create_mock_user("1")

        response = RequestMock.do_request_patch(
            search_operator_rest_views.SearchOperatorDetail.as_view(),
            mock_user,
            param={"pk": 0},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(search_operator_api, "get_by_id")
    @patch.object(SearchOperatorSerializer, "is_valid")
    @patch.object(SearchOperatorSerializer, "save")
    @patch.object(SearchOperatorSerializer, "data")
    def test_staff_returns_http_200(
        self,
        mock_search_operator_serializer_data,
        mock_search_operator_serializer_save,
        mock_search_operator_serializer_is_valid,
        mock_search_operator_api_get_by_id,
    ):
        """test_staff_returns_http_200"""

        mock_search_operator_serializer_data.return_value = {}
        mock_search_operator_serializer_save.return_value = None
        mock_search_operator_serializer_is_valid.return_value = True
        mock_search_operator_api_get_by_id.return_value = None
        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_patch(
            search_operator_rest_views.SearchOperatorDetail.as_view(),
            mock_user,
            param={"pk": 0},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestSearchOperatorDetailDelete(SimpleTestCase):
    """Test Search Operator Detail Delete"""

    def test_anonymous_returns_http_403(self):
        """test_anonymous_returns_http_403"""
        response = RequestMock.do_request_delete(
            search_operator_rest_views.SearchOperatorDetail.as_view(),
            None,
            param={"pk": 0},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_authenticated_returns_http_403(self):
        """test_authenticated_returns_http_403"""

        mock_user = create_mock_user("1")

        response = RequestMock.do_request_delete(
            search_operator_rest_views.SearchOperatorDetail.as_view(),
            mock_user,
            param={"pk": 0},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(search_operator_api, "get_by_id")
    @patch.object(search_operator_api, "delete")
    def test_staff_returns_http_200(
        self,
        mock_search_operator_api_delete,
        mock_search_operator_api_get_by_id,
    ):
        """test_staff_returns_http_200"""

        mock_search_operator_api_delete.return_value = None
        mock_search_operator_api_get_by_id.return_value = None
        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_delete(
            search_operator_rest_views.SearchOperatorDetail.as_view(),
            mock_user,
            param={"pk": 0},
        )

        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)



""" REST views for the search operators.
"""

from django.utils.decorators import method_decorator
from rest_framework import status
from rest_framework.exceptions import ValidationError
from rest_framework.response import Response
from rest_framework.views import APIView

from core_main_app.commons import exceptions
from core_main_app.utils.decorators import api_staff_member_required
import core_explore_keyword_app.components.search_operator.api as search_operator_api
from core_explore_keyword_app.rest.search_operators.serializers import (
    SearchOperatorSerializer,
)


class SearchOperatorList(APIView):
    """List search operators"""

    def get(self, request):
        """Get all search operators

        Args:
            request: HTTP request

        Returns:

            - code: 200
              content: List of search operators
            - code: 500
              content: Internal server error
        """
        try:
            search_operator_list = search_operator_api.get_all()

            # Serialize object
            serializer = SearchOperatorSerializer(
                search_operator_list, many=True
            )

            # Return response
            return Response(serializer.data, status=status.HTTP_200_OK)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(
                content, status=status.HTTP_500_INTERNAL_SERVER_ERROR
            )

    @method_decorator(api_staff_member_required())
    def post(self, request):
        """Create a search operator

        Parameters:
            {
                "name": "search_operator_name",
                "xpath_list": ["/x/path/one", "/x/path/two"],
            }

        Args:
            request: HTTP request

        Returns:

            - code: 201
              content: Created search operator
            - code: 400
              content: Validation error / not unique / model error
            - code: 500
              content: Internal server error
        """
        try:
            # Build serializer
            serializer = SearchOperatorSerializer(data=request.data)

            # Validate data
            serializer.is_valid(raise_exception=True)

            # Save data
            serializer.save()

            return Response(serializer.data, status=status.HTTP_201_CREATED)
        except ValidationError as validation_exception:
            content = {"message": validation_exception.detail}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except exceptions.ModelError as model_exception:
            content = {"message": str(model_exception)}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except exceptions.NotUniqueError as not_unique_error:
            content = {"message": str(not_unique_error)}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except KeyError as key_error:
            content = {"message": str(key_error)}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except exceptions.ApiError as api_exception:
            content = {"message": str(api_exception)}
            return Response(
                content, status=status.HTTP_500_INTERNAL_SERVER_ERROR
            )
        except Exception as exception:
            content = {"message": str(exception)}
            return Response(
                content, status=status.HTTP_500_INTERNAL_SERVER_ERROR
            )


class SearchOperatorDetail(APIView):
    """Search operator detail"""

    def get(self, request, pk):
        """Retrieve search operator from database

        Args:

            request: HTTP request
            pk: ObjectId

        Returns:
            SearchOperator
        """
        try:
            # Get object
            search_operator = search_operator_api.get_by_id(pk)

            # Serialize object
            serializer = SearchOperatorSerializer(search_operator)

            # Return response
            return Response(serializer.data)
        except exceptions.DoesNotExist:
            content = {"message": "Search operator not found."}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(
                content, status=status.HTTP_500_INTERNAL_SERVER_ERROR
            )

    @method_decorator(api_staff_member_required())
    def patch(self, request, pk):
        """Update a search operator

        Parameters:
            {
                "name": "new_name",
                "xpath_list": ["/x/path/one/new", "/x/path/two/new"]
            }

        Args:
            request: HTTP request
            pk: ObjectId

        Returns:
            - code: 200
              content: Updated data
            - code: 400
              content: Validation error
            - code: 404
              content: Object was not found
            - code: 500
              content: Internal server error
        """
        try:
            # Get object
            search_operator = search_operator_api.get_by_id(pk)

            # Build serializer
            search_operator_serializer = SearchOperatorSerializer(
                instance=search_operator, data=request.data, partial=True
            )

            # Validate and save search operator
            search_operator_serializer.is_valid(raise_exception=True)
            search_operator_serializer.save()

            return Response(
                search_operator_serializer.data, status=status.HTTP_200_OK
            )
        except ValidationError as validation_exception:
            content = {"message": validation_exception.detail}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except exceptions.DoesNotExist:
            content = {"message": "Search operator not found."}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(
                content, status=status.HTTP_500_INTERNAL_SERVER_ERROR
            )

    @method_decorator(api_staff_member_required())
    def delete(self, request, pk):
        """Delete a search operator

        Args:
            request: HTTP request
            pk: ObjectId

        Returns:
            - code: 204
              content: Deletion success
            - code: 403
              content: Authentication error
            - code: 404
              content: Object was not found
            - code: 500
              content: Internal server error
        """
        try:
            # Get object
            search_operator = search_operator_api.get_by_id(pk)

            # delete object
            search_operator_api.delete(search_operator)

            # Return response
            return Response(status=status.HTTP_204_NO_CONTENT)
        except exceptions.DoesNotExist:
            content = {"message": "Workspace not found."}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(
                content, status=status.HTTP_500_INTERNAL_SERVER_ERROR
            )



""" Test units
"""

from unittest import TestCase
from unittest.mock import patch

from core_explore_keyword_app.templatetags.core_explore_keyword_app_search_bar import (
    show_search_bar,
)
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import create_mock_request


class TestSearchBar(TestCase):
    """TestSearchBar"""

    @patch("core_explore_common_app.components.query.api.upsert")
    @patch(
        "core_explore_common_app.components.query.api.set_visibility_to_query"
    )
    @patch(
        "core_explore_common_app.components.query.api.add_local_data_source"
    )
    def test_search_bar(
        self,
        mock_add_local_data_source,
        mock_set_visibility_to_query,
        mock_upsert,
    ):
        mock_user = create_mock_user("1")
        mock_request = create_mock_request(user=mock_user)
        mock_context = {"request": mock_request}
        mock_add_local_data_source.return_value = None
        mock_set_visibility_to_query.return_value = None
        mock_upsert.return_value = None

        response = show_search_bar(context=mock_context)

        self.assertTrue("search_form" in response["data"])
        self.assertTrue(mock_add_local_data_source.called)
        self.assertTrue(mock_set_visibility_to_query.called)
        self.assertTrue(mock_upsert.called)



""" Migrations
 """

# Generated by Django 3.2 on 2021-10-08 20:54
import core_main_app.utils.validation.regex_validation
import core_main_app.utils.validation.xpath_validation
from django.db import migrations, models


class Migration(migrations.Migration):
    """Migration"""

    initial = True

    dependencies = [
        ("core_main_app", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="ExploreKeyword",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
            ],
            options={
                "verbose_name": "core_explore_keyword_app",
                "permissions": (
                    ("access_explore_keyword", "Can access explore keyword"),
                ),
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="SearchOperator",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "name",
                    models.CharField(
                        max_length=200,
                        unique=True,
                        validators=[
                            core_main_app.utils.validation.regex_validation.validate_alphanum
                        ],
                    ),
                ),
                (
                    "xpath_list",
                    models.JSONField(
                        unique=True,
                        validators=[
                            core_main_app.utils.validation.xpath_validation.validate_xpath_list
                        ],
                    ),
                ),
                ("dot_notation_list", models.JSONField(unique=True)),
            ],
        ),
        migrations.CreateModel(
            name="PersistentQueryKeyword",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("user_id", models.CharField(max_length=200)),
                ("content", models.TextField(blank=True, null=True)),
                ("data_sources", models.JSONField(blank=True, default=list)),
                ("creation_date", models.DateTimeField(auto_now_add=True)),
                (
                    "name",
                    models.CharField(
                        blank=True, max_length=200, null=True, unique=True
                    ),
                ),
                (
                    "templates",
                    models.ManyToManyField(
                        blank=True, default=[], to="core_main_app.Template"
                    ),
                ),
            ],
            options={
                "abstract": False,
                "verbose_name": "Persistent Query by Keyword",
                "verbose_name_plural": "Persistent Queries by Keyword",
            },
        ),
    ]



""" Abstract Keyword Search Extras
"""

from abc import ABCMeta, abstractmethod


class AbstractKeywordSearchExtras(metaclass=ABCMeta):
    """Abstract Keyword Search Extras"""

    @staticmethod
    @abstractmethod
    def get_extra_html():
        """get_extra_html"""
        raise NotImplementedError("_get_extra_html method is not implemented.")

    @staticmethod
    @abstractmethod
    def get_extra_js():
        """get_extra_js"""
        raise NotImplementedError("_get_extra_js method is not implemented.")

    @staticmethod
    @abstractmethod
    def get_extra_css():
        """get_extra_css"""
        raise NotImplementedError("_get_extra_css method is not implemented.")



""" Url router for administration views
"""

from django.contrib import admin
from django.contrib.admin.views.decorators import staff_member_required
from django.urls import re_path

from core_main_app.admin import core_admin_site
from core_explore_keyword_app.components.persistent_query_keyword.admin_site import (
    CustomPersistentQueryKeywordAdmin,
)
from core_explore_keyword_app.components.persistent_query_keyword.models import (
    PersistentQueryKeyword,
)
from core_explore_keyword_app.components.search_operator.admin_site import (
    CustomSearchOperatorAdmin,
)
from core_explore_keyword_app.components.search_operator.models import (
    SearchOperator,
)
from core_explore_keyword_app.views.admin import ajax as admin_ajax
from core_explore_keyword_app.views.admin import views as admin_views

admin_urls = [
    re_path(
        r"^operators/$",
        staff_member_required(admin_views.ListSearchOperatorsView.as_view()),
        name="core_explore_keyword_app_list_search_operators",
    ),
    re_path(
        r"^operators/edit$",
        staff_member_required(
            admin_ajax.SearchOperatorConfigModalView.as_view()
        ),
        name="core_explore_keyword_app_search_operator_config_modal",
    ),
    re_path(
        r"^operators/delete$",
        staff_member_required(
            admin_ajax.SearchOperatorDeleteModalView.as_view()
        ),
        name="core_explore_keyword_app_search_operator_delete_modal",
    ),
]

admin.site.register(SearchOperator, CustomSearchOperatorAdmin)
admin.site.register(PersistentQueryKeyword, CustomPersistentQueryKeywordAdmin)
urls = core_admin_site.get_urls()
core_admin_site.get_urls = lambda: admin_urls + urls



""" Custom admin site for the Search Operator model
"""

from django.contrib import admin


class CustomSearchOperatorAdmin(admin.ModelAdmin):
    """CustomSearchOperatorAdmin"""

    readonly_fields = ["xpath_list", "dot_notation_list"]

    def has_add_permission(self, request, obj=None):
        """Prevent from manually adding Search Operators"""
        return False



"""  Menu configuration for core_main_app. Upon installation of the app the following
menus are displayed:

  * User menu

    * `core_explore_keyword_app.settings.EXPLORE_MENU_NAME`

  * Admin menu

    * Explore by keyword
        * Search operators
"""

from django.urls import reverse
from menu import Menu, MenuItem

from core_explore_keyword_app.settings import EXPLORE_MENU_NAME

# FIXME: CHECK AUTHENTICATION !
Menu.add_item(
    "main",
    MenuItem(EXPLORE_MENU_NAME, reverse("core_explore_keyword_app_search")),
)

explore_by_keyword_children = (
    MenuItem(
        "Search Operators",
        reverse("core-admin:core_explore_keyword_app_list_search_operators"),
        icon="code",
    ),
)

Menu.add_item(
    "admin",
    MenuItem("EXPLORE BY KEYWORD", None, children=explore_by_keyword_children),
)



"""Explore Keyword models
"""

from django.db import models

from core_main_app.permissions.utils import get_formatted_name
from core_explore_keyword_app.permissions import rights


class ExploreKeyword(models.Model):
    """Explore Keyword"""

    class Meta:
        """Meta"""

        verbose_name = "core_explore_keyword_app"
        default_permissions = ()
        permissions = (
            (
                rights.EXPLORE_KEYWORD_ACCESS,
                get_formatted_name(rights.EXPLORE_KEYWORD_ACCESS),
            ),
        )



""" Unit tests for user-side AJAX calls
"""

import json
from unittest import TestCase
from unittest.mock import MagicMock, patch

from django.test import RequestFactory
from rest_framework import status

from core_explore_common_app.constants import LOCAL_QUERY_NAME
from core_explore_keyword_app.views.user.ajax import (
    SuggestionsKeywordSearchView,
    _get_local_data_source,
)
from core_main_app.commons.exceptions import QueryError, DoesNotExist
from core_main_app.settings import SERVER_URI
from core_main_app.utils.tests_tools.MockUser import create_mock_user


class TestSuggestionsKeywordSearchViewPost(TestCase):
    """Test SuggestionsKeywordSearchView post method"""

    @classmethod
    def setUpClass(cls) -> None:
        """setUpClass"""
        cls.data = {
            "keywords": "mock_keyword",
            "query_id": 1,
            "user_id": 1,
            "term": "mock_term",
        }

        cls.user = create_mock_user(user_id=1)
        cls.view = "core_explore_keyword_suggestions"

        cls.factory = RequestFactory()

    def setUp(self) -> None:
        """setUp"""
        self.user.has_perm = MagicMock()
        self.user.has_perm.return_value = True

    def _send_post_request(self):
        """_send_post_request"""
        request = self.factory.post(self.view, data=self.data)
        request.user = self.user

        return SuggestionsKeywordSearchView().post(request)

    @patch("core_explore_keyword_app.views.user.ajax.KeywordForm.is_valid")
    def test_invalid_form_returns_400(self, mock_keyword_form_is_valid):
        """test_invalid_form_returns_400"""
        mock_keyword_form_is_valid.return_value = False

        response = self._send_post_request()

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    @patch("core_explore_keyword_app.views.user.ajax.sanitize_value")
    @patch("core_explore_keyword_app.views.user.ajax.KeywordForm")
    def test_invalid_term_returns_400(
        self, mock_keyword_form, mock_sanitize_value
    ):
        """test_invalid_term_returns_400"""
        mock_keyword_form.return_value = MagicMock()
        mock_sanitize_value.side_effect = QueryError("mock_error")

        response = self._send_post_request()

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    @patch(
        "core_explore_keyword_app.views.user.ajax.template_version_manager_api.get_by_id_list"
    )
    @patch("core_explore_keyword_app.views.user.ajax.sanitize_value")
    @patch("core_explore_keyword_app.views.user.ajax.KeywordForm")
    def test_template_not_found_returns_400(
        self, mock_keyword_form, mock_sanitize_value, mock_template_get_by_id
    ):
        """test_template_not_found_returns_400"""
        mock_keyword_form.return_value = MagicMock()
        mock_sanitize_value.return_value = None
        mock_template_get_by_id.side_effect = DoesNotExist(
            "mock_does_not_exist_error"
        )

        response = self._send_post_request()

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    @patch("core_explore_keyword_app.views.user.ajax.query_api.get_by_id")
    @patch(
        "core_explore_keyword_app.views.user.ajax.template_version_manager_api.get_by_id_list"
    )
    @patch("core_explore_keyword_app.views.user.ajax.sanitize_value")
    @patch("core_explore_keyword_app.views.user.ajax.KeywordForm")
    def test_query_not_found_returns_400(
        self,
        mock_keyword_form,
        mock_sanitize_value,
        mock_template_get_by_id,
        mock_query_get_by_id,
    ):
        """test_query_not_found_returns_400"""
        mock_keyword_form.return_value = MagicMock()
        mock_sanitize_value.return_value = None
        mock_template_get_by_id.return_value = []
        mock_query_get_by_id.side_effect = DoesNotExist(
            "mock_does_not_exist_error"
        )

        response = self._send_post_request()

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    @patch("core_explore_keyword_app.views.user.ajax._get_local_data_source")
    @patch("core_explore_keyword_app.views.user.ajax.query_api.get_by_id")
    @patch(
        "core_explore_keyword_app.views.user.ajax.template_version_manager_api.get_by_id_list"
    )
    @patch("core_explore_keyword_app.views.user.ajax.sanitize_value")
    @patch("core_explore_keyword_app.views.user.ajax.KeywordForm")
    def test_get_local_data_source_fails_returns_400(
        self,
        mock_keyword_form,
        mock_sanitize_value,
        mock_template_get_by_id,
        mock_query_get_by_id,
        mock_get_local_data_source,
    ):
        """test_get_local_data_source_fails_returns_400"""
        mock_keyword_form.return_value = MagicMock()
        mock_sanitize_value.return_value = None
        mock_template_get_by_id.return_value = []
        mock_query_get_by_id.return_value = MagicMock()
        mock_get_local_data_source.side_effect = Exception(
            "mock_get_local_data_source_exception"
        )

        response = self._send_post_request()

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    @patch(
        "core_explore_keyword_app.views.user.ajax.SuggestionsKeywordSearchView._get_query_prepared"
    )
    @patch("core_explore_keyword_app.views.user.ajax._get_local_data_source")
    @patch("core_explore_keyword_app.views.user.ajax.query_api.get_by_id")
    @patch(
        "core_explore_keyword_app.views.user.ajax.template_version_manager_api.get_by_id_list"
    )
    @patch("core_explore_keyword_app.views.user.ajax.sanitize_value")
    @patch("core_explore_keyword_app.views.user.ajax.KeywordForm")
    def test_get_query_prepared_fails_returns_400(
        self,
        mock_keyword_form,
        mock_sanitize_value,
        mock_template_get_by_id,
        mock_query_get_by_id,
        mock_get_local_data_source,
        mock_get_query_prepared,
    ):
        """test_get_query_prepared_fails_returns_400"""
        mock_keyword_form.return_value = MagicMock()
        mock_sanitize_value.return_value = None
        mock_template_get_by_id.return_value = []
        mock_query_get_by_id.return_value = MagicMock()
        mock_get_local_data_source.return_value = {
            "query_options": {},
            "order_by_field": "title",
        }
        mock_get_query_prepared.side_effect = Exception(
            "mock_get_query_prepared_exception"
        )

        response = self._send_post_request()

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    @patch(
        "core_explore_keyword_app.views.user.ajax.SuggestionsKeywordSearchView._extract_suggestion_from_results"
    )
    @patch(
        "core_explore_keyword_app.views.user.ajax.SuggestionsKeywordSearchView._get_query_prepared"
    )
    @patch("core_explore_keyword_app.views.user.ajax._get_local_data_source")
    @patch("core_explore_keyword_app.views.user.ajax.query_api.get_by_id")
    @patch(
        "core_explore_keyword_app.views.user.ajax.template_version_manager_api.get_by_id_list"
    )
    @patch("core_explore_keyword_app.views.user.ajax.sanitize_value")
    @patch("core_explore_keyword_app.views.user.ajax.KeywordForm")
    def test_extract_suggestions_fails_returns_400(
        self,
        mock_keyword_form,
        mock_sanitize_value,
        mock_template_get_by_id,
        mock_query_get_by_id,
        mock_get_local_data_source,
        mock_get_query_prepared,
        mock_extract_suggestion_from_results,
    ):
        """test_extract_suggestions_fails_returns_400"""
        mock_keyword_form.return_value = MagicMock()
        mock_sanitize_value.return_value = None
        mock_template_get_by_id.return_value = []
        mock_query_get_by_id.return_value = MagicMock()
        mock_get_local_data_source.return_value = {
            "query_options": {},
            "order_by_field": "title",
        }
        mock_get_query_prepared.return_value = MagicMock()
        mock_extract_suggestion_from_results.side_effect = Exception(
            "mock_extract_suggestion_from_results_exception"
        )

        response = self._send_post_request()

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    @patch(
        "core_explore_keyword_app.views.user.ajax.SuggestionsKeywordSearchView._extract_suggestion_from_results"
    )
    @patch("core_main_app.components.data.api.execute_json_query")
    @patch(
        "core_explore_keyword_app.views.user.ajax.SuggestionsKeywordSearchView._get_query_prepared"
    )
    @patch("core_explore_keyword_app.views.user.ajax._get_local_data_source")
    @patch("core_explore_keyword_app.views.user.ajax.query_api.get_by_id")
    @patch(
        "core_explore_keyword_app.views.user.ajax.template_version_manager_api.get_by_id_list"
    )
    @patch("core_explore_keyword_app.views.user.ajax.sanitize_value")
    @patch("core_explore_keyword_app.views.user.ajax.KeywordForm")
    def test_empty_dict_results_returns_200(
        self,
        mock_keyword_form,
        mock_sanitize_value,
        mock_template_get_by_id,
        mock_query_get_by_id,
        mock_get_local_data_source,
        mock_get_query_prepared,
        mock_execute_local_query,
        mock_extract_suggestion_from_results,
    ):
        """test_empty_dict_results_returns_200"""
        mock_keyword_form.return_value = MagicMock()
        mock_sanitize_value.return_value = None
        mock_template_get_by_id.return_value = []
        mock_query_get_by_id.return_value = MagicMock()
        mock_get_local_data_source.return_value = {
            "query_options": {},
            "order_by_field": "title",
        }
        mock_get_query_prepared.return_value = MagicMock()
        mock_execute_local_query.return_value = MagicMock()
        mock_extract_suggestion_from_results.return_value = None

        response = self._send_post_request()

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch(
        "core_explore_keyword_app.views.user.ajax.SuggestionsKeywordSearchView._extract_suggestion_from_results"
    )
    @patch("core_main_app.components.data.api.execute_json_query")
    @patch(
        "core_explore_keyword_app.views.user.ajax.SuggestionsKeywordSearchView._get_query_prepared"
    )
    @patch("core_explore_keyword_app.views.user.ajax._get_local_data_source")
    @patch("core_explore_keyword_app.views.user.ajax.query_api.get_by_id")
    @patch(
        "core_explore_keyword_app.views.user.ajax.template_version_manager_api.get_by_id_list"
    )
    @patch("core_explore_keyword_app.views.user.ajax.sanitize_value")
    @patch("core_explore_keyword_app.views.user.ajax.KeywordForm")
    def test_dict_results_returns_suggestions(
        self,
        mock_keyword_form,
        mock_sanitize_value,
        mock_template_get_by_id,
        mock_query_get_by_id,
        mock_get_local_data_source,
        mock_get_query_prepared,
        mock_execute_json_query,
        mock_extract_suggestion_from_results,
    ):
        """test_dict_results_returns_suggestions"""
        mock_keyword_form.return_value = MagicMock()
        mock_sanitize_value.return_value = None
        mock_template_get_by_id.return_value = []
        mock_query_get_by_id.return_value = MagicMock()
        mock_get_local_data_source.return_value = {
            "query_options": {},
            "order_by_field": "title",
        }
        mock_get_query_prepared.return_value = MagicMock()
        mock_execute_json_query.return_value = MagicMock()
        mock_extract_suggestion_from_results.return_value = None

        response = self._send_post_request()

        self.assertEqual(json.loads(response.content), {"suggestions": []})


class TestGetLocalDataSource(TestCase):
    """TestGetLocalDataSource"""

    def test_get_local_data_source(self):
        """test_get_local_data_source

        Returns:

        """
        mock_query = MagicMock()
        mock_query.data_sources = [
            {"name": LOCAL_QUERY_NAME, "url_query": SERVER_URI}
        ]
        data_source = _get_local_data_source(mock_query)
        self.assertIsNotNone(data_source)

    def test_get_local_data_source_returns_none(self):
        """test_get_local_data_source_returns_none

        Returns:

        """
        mock_query = MagicMock()
        mock_query.data_sources = [{"name": "test", "url_query": SERVER_URI}]
        data_source = _get_local_data_source(mock_query)
        self.assertIsNone(data_source)



""" Admin forms for Search Operators.
"""

from django import forms
from django.forms import ModelForm

from core_main_app.commons import exceptions as core_main_app_exceptions
from core_main_app.utils.xml import validate_xpath
from core_explore_keyword_app.components.search_operator.models import (
    SearchOperator,
)
from core_explore_keyword_app.components.search_operator import (
    api as search_operator_api,
)


class SearchOperatorForm(ModelForm):
    """Main form to edit search operators."""

    document_id = forms.CharField(required=False, widget=forms.HiddenInput())
    xpath_list = forms.CharField(
        help_text="Enter one xpath per line.",
        required=True,
        widget=forms.Textarea(
            attrs={"placeholder": "/path/to/xpath", "class": "form-control"}
        ),
    )

    class Meta:
        """Meta"""

        model = SearchOperator
        fields = ["name"]

        widgets = {
            "name": forms.TextInput(
                attrs={"placeholder": "Operator name", "class": "form-control"}
            )
        }

    def clean_xpath_list(self):
        """Validate the list of xpath.

        Returns:
        """
        data = self.cleaned_data["xpath_list"].split("\n")
        line_xpath = 0

        for xpath in data:
            line_xpath += 1
            try:
                validate_xpath(xpath)
            except core_main_app_exceptions.XMLError as exception:
                raise forms.ValidationError(
                    "XPath syntax error (line %d): %s"
                    % (line_xpath, str(exception))
                )

        return self.cleaned_data["xpath_list"]

    def save(self, commit=True):
        instance = super().save(commit=False)
        instance.xpath_list = [
            xpath.strip()
            for xpath in self.cleaned_data["xpath_list"].split("\n")
        ]
        if commit:
            for xpath in instance.xpath_list:
                # check if xpath already exist
                if any(
                    xpath in sub_xpath_list
                    for sub_xpath_list in search_operator_api.get_all_xpath_except_xpath(
                        instance
                    )
                ):
                    raise core_main_app_exceptions.ApiError(
                        "Xpath already exists"
                    )
            return search_operator_api.upsert(instance)
        return instance



""" Search Operator model
"""

from django.core.exceptions import ObjectDoesNotExist
from django.db import IntegrityError
from django.db import models

from core_main_app.commons import exceptions
from core_main_app.commons.exceptions import ModelError
from core_main_app.utils.validation.regex_validation import validate_alphanum
from core_main_app.utils.validation.xpath_validation import validate_xpath_list


class SearchOperator(models.Model):
    """Search Operator model"""

    name = models.CharField(
        blank=False,
        unique=True,
        validators=[validate_alphanum],
        max_length=200,
    )
    xpath_list = models.JSONField(
        blank=False, unique=True, validators=[validate_xpath_list]
    )
    dot_notation_list = models.JSONField(blank=False, unique=True)

    @staticmethod
    def get_all():
        """Retrieve all search operators.

        Returns:
        """
        return SearchOperator.objects.all()

    @staticmethod
    def get_by_id(operator_id):
        """Retrieve a search operator with a given id.

        Args:
            operator_id:

        Returns:
        """
        try:
            return SearchOperator.objects.get(pk=operator_id)
        except ModelError as validation_error:
            raise exceptions.ModelError(str(validation_error))
        except ObjectDoesNotExist as does_not_exist:
            raise exceptions.DoesNotExist(str(does_not_exist))

    @staticmethod
    def get_by_name(operator_name):
        """Retrieve all search operators with a given name.

        Args:
            operator_name:

        Returns:
        """
        try:
            return SearchOperator.objects.get(name=operator_name)
        except ObjectDoesNotExist as does_not_exist:
            raise exceptions.DoesNotExist(str(does_not_exist))

    @staticmethod
    def get_by_dot_notation_list(operator_dot_notation_list):
        """Retrieve all search operators with a given dot_notation list.

        Args:
            operator_dot_notation_list:

        Returns:
        """
        try:
            return SearchOperator.objects.get(
                dot_notation_list=operator_dot_notation_list
            )
        except ObjectDoesNotExist as does_not_exist:
            raise exceptions.DoesNotExist(str(does_not_exist))

    @staticmethod
    def get_all_xpath_list_except_xpath(operator_id):
        """Retrieve all xpath

        Args:

        Returns:
        """
        return (
            SearchOperator.objects.exclude(id=operator_id)
            .all()
            .values_list("xpath_list", flat=True)
        )

    def save_object(self):
        """Upsert a search operator and handle possible issues.

        Returns:
            SearchOperator
        """
        try:
            return self.save()
        except IntegrityError as not_unique_error:
            raise exceptions.NotUniqueError(str(not_unique_error))
        except Exception as exception:
            raise exceptions.ModelError(str(exception))

    def __str__(self):
        """Search Operator object as string

        Returns:

        """
        return self.name



""" Unit tests for PersistentQueryKeyword.
"""

from unittest import TestCase, mock

from unittest.mock import patch

from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.commons import exceptions
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_explore_keyword_app.components.persistent_query_keyword import (
    api as persistent_query_keyword_api,
)
from core_explore_keyword_app.components.persistent_query_keyword.models import (
    PersistentQueryKeyword,
)
from core_explore_keyword_app.views.user import views as keyword_search_views


class TestPersistentQueryKeywordGetById(TestCase):
    """Test Persistent Query Keyword Get By Id"""

    @patch.object(PersistentQueryKeyword, "get_by_id")
    def test_persistent_query_keyword_get_by_id_return_data_if_found(
        self, mock_get_by_id
    ):
        """test_persistent_query_keyword_get_by_id_return_data_if_found"""

        # Arrange
        expected_result = PersistentQueryKeyword(user_id="1")
        mock_get_by_id.return_value = expected_result
        mock_user = create_mock_user("1")

        # Act # Assert
        self.assertEqual(
            persistent_query_keyword_api.get_by_id("mock_id", mock_user),
            expected_result,
        )

    def test_persistent_query_keyword_get_by_id_raises_model_error_if_not_found(
        self,
    ):
        """test_persistent_query_keyword_get_by_id_raises_model_error_if_not_found"""

        # Arrange
        mock_user = create_mock_user("1")

        # Act # Assert
        with self.assertRaises(exceptions.ModelError):
            persistent_query_keyword_api.get_by_id("mock_id", mock_user)

    @patch.object(PersistentQueryKeyword, "get_by_id")
    def test_persistent_query_keyword_get_by_id_raises_does_not_exist_error_if_not_found(
        self, mock_get_by_id
    ):
        """test_persistent_query_keyword_get_by_id_raises_does_not_exist_error_if_not_found"""

        # Arrange
        mock_get_by_id.side_effect = exceptions.DoesNotExist(
            message="mock error"
        )
        mock_user = create_mock_user("1")

        # Act # Assert
        with self.assertRaises(exceptions.DoesNotExist):
            persistent_query_keyword_api.get_by_id("mock_id", mock_user)


class TestsPersistentQueryKeywordGetByName(TestCase):
    """Tests Persistent Query Keyword Get By Name"""

    @mock.patch.object(PersistentQueryKeyword, "get_by_name")
    def test_persistent_query_keyword_get_by_name_return_data_if_found(
        self, mock_get_by_name
    ):
        """test_persistent_query_keyword_get_by_name_return_data_if_found"""

        # Arrange
        expected_result = PersistentQueryKeyword(user_id="1")
        mock_get_by_name.return_value = expected_result
        mock_user = create_mock_user("1")

        # Act # Assert
        self.assertEqual(
            persistent_query_keyword_api.get_by_name("mock_name", mock_user),
            expected_result,
        )

    @patch.object(PersistentQueryKeyword, "get_by_name")
    def test_persistent_query_keyword_get_by_name_raises_does_not_exist_error_if_not_found(
        self, mock_get_by_name
    ):
        """test_persistent_query_keyword_get_by_name_raises_does_not_exist_error_if_not_found"""

        # Arrange
        mock_get_by_name.side_effect = exceptions.DoesNotExist(
            message="mock error"
        )
        mock_user = create_mock_user("1")

        # Act # Assert
        with self.assertRaises(exceptions.DoesNotExist):
            persistent_query_keyword_api.get_by_name("mock_id", mock_user)


class TestsPersistentQueryKeywordUpsert(TestCase):
    """Tests Persistent Query Keyword Upsert"""

    def setUp(self) -> None:
        self.mock_persistent_query_keyword = PersistentQueryKeyword(
            user_id="1",
            name="mock_keyword",
            content={"content_test"},
            data_sources=[],
        )

    @patch.object(PersistentQueryKeyword, "save")
    def test_persistent_query_keyword_upsert_return_data(self, mock_save):
        """test_persistent_query_keyword_upsert_return_data"""

        # Arrange
        mock_save.return_value = self.mock_persistent_query_keyword
        mock_user = create_mock_user("1")

        # Act
        result = persistent_query_keyword_api.upsert(
            self.mock_persistent_query_keyword, mock_user
        )

        # Assert
        self.assertIsInstance(result, PersistentQueryKeyword)


class TestsPersistentQueryKeywordDelete(TestCase):
    """Tests Persistent Query Keyword Delete"""

    @patch.object(PersistentQueryKeyword, "delete")
    def test_returns_no_error(self, mock_delete):
        """test_returns_no_error"""

        # Arrange
        mock_delete.return_value = None
        mock_user = create_mock_user("1")

        # Act # Assert
        self.assertEqual(
            persistent_query_keyword_api.delete(
                PersistentQueryKeyword(user_id="1"), mock_user
            ),
            None,
        )


class TestsPersistentQueryKeywordGetAll(TestCase):
    """Tests Persistent Query Keyword Get All"""

    @patch.object(PersistentQueryKeyword, "get_all")
    def test_returns_no_error(self, mock_get_all):
        """test_returns_no_error"""

        # Arrange
        expected_result = {
            PersistentQueryKeyword(id=1, user_id="1"),
            PersistentQueryKeyword(id=2, user_id="2"),
        }
        mock_get_all.return_value = expected_result

        mock_user = create_mock_user("1", is_superuser=True, is_staff=True)

        # Act # Assert
        self.assertEqual(
            persistent_query_keyword_api.get_all(mock_user), expected_result
        )

    @patch.object(PersistentQueryKeyword, "get_all")
    def test_persistent_query_keyword_get_all_raises_does_not_access_control_error_if_not_admin(
        self, mock_get_all
    ):
        """test_persistent_query_keyword_get_all_raises_does_not_access_control_error_if_not_admin"""

        # Arrange
        mock_get_all.side_effect = AccessControlError
        mock_user = create_mock_user("1")

        # Act # Assert
        with self.assertRaises(AccessControlError):
            persistent_query_keyword_api.get_all(mock_user)


class TestsPersistentQueryKeywordGetAllByUser(TestCase):
    """Tests Persistent Query Keyword Get All By User"""

    @patch.object(PersistentQueryKeyword, "get_all_by_user")
    def test_returns_no_error(self, mock_get_all_by_user):
        """test_returns_no_error"""

        # Arrange
        expected_result = {
            PersistentQueryKeyword(id=1, user_id="1"),
            PersistentQueryKeyword(id=2, user_id="1"),
        }
        mock_get_all_by_user.return_value = expected_result

        mock_user = create_mock_user("1")

        # Act # Assert
        self.assertEqual(
            persistent_query_keyword_api.get_all_by_user(mock_user),
            expected_result,
        )


class TestKeywordSearch(TestCase):
    """Test Keyword Search"""

    def test_build_query_content_with_simple_keywords(self):
        """test_build_query_content_with_simple_keywords"""

        # set query
        initial_keyword_list = ["FirstTest", "SecondTest"]

        # Act
        main_query = keyword_search_views.KeywordSearchView._build_query(
            initial_keyword_list
        )

        # assert
        self.assertEqual(
            main_query,
            '{"$text": {"$search": "\\"FirstTest\\" \\"SecondTest\\""}}',
        )

    def test_build_query_content_with_hyphens(self):
        """test_build_query_content_with_hyphens"""

        # set query
        initial_keyword_list = ["test-hyphens", "more-hyphen-keyword"]

        # Act
        main_query = keyword_search_views.KeywordSearchView._build_query(
            initial_keyword_list
        )

        # assert
        self.assertEqual(
            main_query,
            '{"$text": {"$search": "\\"test-hyphens\\" \\"more-hyphen-keyword\\""}}',
        )

    def test_build_query_content_with_hyphens_and_numbers(self):
        """test_build_query_content_with_hyphens_and_numbers"""

        # set query
        initial_keyword_list = [
            "test-hyphens_number-123",
            "more-hyphen-keyword_number-456",
        ]

        # Act
        main_query = keyword_search_views.KeywordSearchView._build_query(
            initial_keyword_list
        )

        # assert
        self.assertEqual(
            main_query,
            '{"$text": {"$search": "\\"test-hyphens_number-123\\" \\"more-hyphen-keyword_number-456\\""}}',
        )

    def test_build_query_content_with_blank(self):
        """test_build_query_content_with_blank"""

        # set query
        initial_keyword_list = ['"blank between keywords"']

        # Act
        main_query = keyword_search_views.KeywordSearchView._build_query(
            initial_keyword_list
        )

        # assert
        self.assertEqual(
            main_query,
            '{"$text": {"$search": "\\"\\"blank between keywords\\"\\""}}',
        )

    def test_build_query_content_with_blank_and_hyphens(self):
        """test_build_query_content_with_blank_and_hyphens"""

        # set query
        initial_keyword_list = [
            '"blank and-hyphens-between keywords"',
            '"test_-hyphens number12 ? "',
        ]

        # Act
        main_query = keyword_search_views.KeywordSearchView._build_query(
            initial_keyword_list
        )

        # assert
        self.assertEqual(
            main_query,
            '{"$text": {"$search": "\\"\\"blank and-hyphens-between keywords\\"\\" '
            '\\"\\"test_-hyphens number12 ? \\"\\""}}',
        )

    def test_parse_simple_query_content(self):
        """test_parse_simple_query_content"""

        # set query
        query_content = (
            '{"$text": {"$search": "\\"FirstTest\\" \\"SecondTest\\""}}'
        )

        # Act
        keyword_list = keyword_search_views.KeywordSearchView._parse_query(
            query_content
        )

        # assert
        self.assertEqual(keyword_list, "FirstTest,SecondTest")

    def test_parse_query_content_with_hyphens(self):
        """test_parse_query_content_with_hyphens"""

        # set query
        query_content = '{"$text": {"$search": "\\"first-hyphen\\" \\"more-hyphen-keywords\\""}}'

        # Act
        keyword_list = keyword_search_views.KeywordSearchView._parse_query(
            query_content
        )

        # assert
        self.assertEqual(keyword_list, "first-hyphen,more-hyphen-keywords")

    def test_parse_query_content_with_hyphens_and_numbers(self):
        """test_parse_query_content_with_hyphens_and_numbers"""

        # set query
        query_content = '{"$text": {"$search": "\\"number-123\\" \\"more-numbers-456\\" \\"last-number-6\\""}}'

        # Act
        keyword_list = keyword_search_views.KeywordSearchView._parse_query(
            query_content
        )

        # assert
        self.assertEqual(
            keyword_list, "number-123,more-numbers-456,last-number-6"
        )

    def test_parse_query_content_with_blank(self):
        """test_parse_query_content_with_blank"""

        # set query
        query_content = (
            '{"$text": {"$search": "\\"\\"blank between keywords\\"\\""}}'
        )

        # Act
        keyword_list = keyword_search_views.KeywordSearchView._parse_query(
            query_content
        )

        # assert
        self.assertEqual(keyword_list, "blank between keywords")

    def test_parse_query_content_with_blank_and_hyphens(self):
        """test_parse_query_content_with_blank_and_hyphens"""

        # set query
        query_content = (
            '{"$text": {"$search": "\\"\\"blank and-hyphens-between keywords\\"\\" '
            '\\"\\"test_-hyphens number12 ? \\"\\""}}'
        )

        # Act
        keyword_list = keyword_search_views.KeywordSearchView._parse_query(
            query_content
        )

        # assert
        self.assertEqual(
            keyword_list,
            "blank and-hyphens-between keywords,test_-hyphens number12 ? ",
        )



""" Url router for the REST API
"""

from django.urls import re_path
from rest_framework.urlpatterns import format_suffix_patterns

from core_explore_keyword_app.rest.persistent_query_keyword import (
    views as persistent_query_keyword_views,
)
from core_explore_keyword_app.rest.search_operators import (
    views as search_operator_views,
)

urlpatterns = [
    re_path(
        r"^search_operators/$",
        search_operator_views.SearchOperatorList.as_view(),
        name="core_explore_keyword_app_rest_search_operator_list",
    ),
    re_path(
        r"^search_operator/(?P<pk>\w+)/$",
        search_operator_views.SearchOperatorDetail.as_view(),
        name="core_explore_keyword_app_rest_search_operator",
    ),
    re_path(
        r"^admin/persistent_query_keyword/$",
        persistent_query_keyword_views.AdminPersistentQueryKeywordList.as_view(),
        name="core_explore_keyword_app_rest_persistent_query_keyword_admin_list",
    ),
    re_path(
        r"^persistent_query_keyword/$",
        persistent_query_keyword_views.PersistentQueryKeywordList.as_view(),
        name="core_explore_keyword_app_rest_persistent_query_keyword_list",
    ),
    re_path(
        r"^persistent_query_keyword/(?P<pk>\w+)/$",
        persistent_query_keyword_views.PersistentQueryKeywordDetail.as_view(),
        name="core_explore_keyword_app_rest_persistent_query_keyword_detail",
    ),
    re_path(
        r"^persistent_query_keyword/name/(?P<name>\w+)/$",
        persistent_query_keyword_views.PersistentQueryKeywordByName.as_view(),
        name="core_explore_keyword_app_rest_persistent_query_keyword_name",
    ),
]

urlpatterns = format_suffix_patterns(urlpatterns)
